module.exports = (app, router) => {
    const Anime = app.db.models.ani.series;

    router.route('/:id')
        .post(app.auth.token, app.auth.perms('series-submit'), app.auth.permsPass('series-approve'), async (req, res) => {
        let submitting = req.unauthorized; //if user doesn't have series-approve, they still have perms to submit by this point

        /**REQUIRED ITEMS
         * 
         *!Submissions must include a name and id (romaji included)
         * Only altNames, tags, and streaming locations can be omitted
         * 
         * If a request has submissible criteria but not completable
         * criteria, it will be treated as a submission even if the user
         * is authorized to approve series.
         */

        //TODO submit anyways if incomplete but user has approval permissions
        //TODO un-submitted (incomplete) but curated route

        
        if (!req.params.id) {return res.status(400).send("You didn't include an anime ID in your request!");}
        if (await Anime.findOne({id: req.params.id})) {return res.status(400).send("An anime already exists with that ID!");}
        if (
            !req.body //i just ate dinner and i can't even think straight
            || !req.body.name || !req.body.romaji || !req.params.id || !req.authenticatedUser || !req.authenticatedUser.id
            || !req.body.name.match(/^[\w_\- ]+$/gm) || req.body.name.length > 150
            || req.body.romaji.length > 150
            || !req.params.id.match(/^[\w_\-]+$/gm) || req.params.id.length > 25
        ) {return res.status(400).send("The server cannot accept your request as your body is missing fields or is malformed. Ensure fields aren't too long and that they don't contain illegal characters.");}
        
        let series = new Anime({
            id: req.params.id,
            numericalId: app.cache.seriesCount + 1,
            name: req.body.name,
            romaji: req.body.romaji,
            synopsis: {
                by: req.body.synopsis ? req.authenticatedUser.id : null,
                synopsis: req.body.synopsis || "A synopsis is not yet available for this series..."
            },
            meta: {
                submitted: submitting ? req.authenticatedUser.id : false, //TODO make sure to update submitted status
                creator: req.authenticatedUser.id,
                edits: [{
                    user: req.authenticatedUser.id,
                    action: 'Submitted series',
                    timestamp: new Date().getTime()
                }]
            },
            genres: req.body.genres && Array.isArray(req.body.genres) && req.body.genres.length ? req.body.genres : []
        });
        if (!req.body.synopsis || !req.body.genres || !Array.isArray(req.body.genres) || !req.body.genres.length) {series.meta.submitted = req.authenticatedUser.id;}
        return series.save().then(async () => {
            app.cache.series[series.id] = {
                id: series.id,
                name: series.name,
                romaji: series.romaji,
                kanji: series.kanji,
                altNames: series.altNames,
                genres: series.genres, 
                tags: series.tags
            };
            return res.send(`Your series was successfully ${series.meta.submitted ? 'submitted' : "added"}.`);
        }).catch((e) => {console.error(e); res.status(500).send("There was an error trying to process your request. It's likely that our database found something wrong with your body fields, and the server didn't realize. Check your request and try again.");});
        //TODO remove console error
    });
};